@@ -153,7 +153,7 @@ class MarionetteJSTestCase(CommonTestCase):
         CommonTestCase.__init__(self, methodName)
 
     @classmethod
-    def add_tests_to_suite(cls, mod_name, filepath, suite, testloader, marionette):
+    def add_tests_to_suite(cls, mod_name, filepath, suite, testloader, marionette, testvars):
         suite.addTest(cls(weakref.ref(marionette), jsFile=filepath))
 
     def runTest(self):
@@ -161,16 +161,6 @@ class MarionetteJSTestCase(CommonTestCase):
             self.marionette.start_session()
         self.marionette.execute_script("log('TEST-START: %s');" % self.jsFile.replace('\\', '\\\\'))
 
-        self.marionette.set_context(self.marionette.CONTEXT_CHROME)
-        self.marionette.set_script_timeout(30000)
-        self.marionette.execute_async_script("""
-waitFor(
-    function() { marionetteScriptFinished(true); },
-    function() { return isSystemMessageListenerReady(); }
-);
-            """)
-        self.marionette.set_context(self.marionette.CONTEXT_CONTENT)
-
         f = open(self.jsFile, 'r')
         js = f.read()
         args = []
